<?php

namespace SM\Factory;

use Doctrine\ORM\EntityManager;
use \Teknisa\Model\Traits\TQuery;

class Sm00005000TabParametrizacoesEmailFactory
{
    use TQuery;

    protected $connection;

    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    public function getParametrosEmailFilial()
    {
        $sql = "SELECT
                    EMAIL.NRPARAMEMAILFILIAL,
                    EMAIL.CDFILIAL,
                    EMAIL.DSPARAMEMAILFILIAL,
                    EMAIL.DSREMETENTEEMAIL,
                    EMAIL.DSASSUNTOEMAIL,
                    DBMS_LOB.SUBSTR(EMAIL.DSCORPOEMAIL, 4000, 1) AS DSCORPOEMAIL, -- Conversão do CLOB para VARCHAR2
                    EMAIL.NRORG,
                    EMAIL.DTINCLUSAO,
                    EMAIL.DTULTATU,
                    EMAIL.NRORGINCLUSAO,
                    EMAIL.CDOPERINCLUSAO,
                    EMAIL.NRORGULTATU,
                    EMAIL.CDOPERULTATU,
                    EMAIL.IDATIVO,
                    EMAIL.NRTIPOORCAMENTO,
                    LISTAGG(ESCOPO.NMTIPOORCAMENTO, ', ') WITHIN GROUP(
                    ORDER BY
                        ESCOPO.NRTIPOORCAMENTO
                    )                AS NMTIPOORCAMENTO,
                    FILIAL.NMFILIAL
                FROM
                    FCT_PARAMEMAILFILIAL EMAIL
                    LEFT JOIN FILIAL ON EMAIL.CDFILIAL = FILIAL.CDFILIAL
                    LEFT JOIN FCT_TIPOORCAMENTO ESCOPO ON ((INSTR('_'||EMAIL.NRTIPOORCAMENTO||'_','_'||TO_CHAR(ESCOPO.NRTIPOORCAMENTO)||'_') > 0)) 
                GROUP BY
                    EMAIL.NRPARAMEMAILFILIAL,
                    EMAIL.CDFILIAL,
                    EMAIL.DSREMETENTEEMAIL,
                    EMAIL.DSASSUNTOEMAIL,
                    EMAIL.DSPARAMEMAILFILIAL,
                    DBMS_LOB.SUBSTR(EMAIL.DSCORPOEMAIL, 4000, 1), -- Conversão também no GROUP BY
                    EMAIL.NRORG,
                    EMAIL.DTINCLUSAO,
                    EMAIL.DTULTATU,
                    EMAIL.NRORGINCLUSAO,
                    EMAIL.CDOPERINCLUSAO,
                    EMAIL.NRORGULTATU,
                    EMAIL.CDOPERULTATU,
                    EMAIL.IDATIVO,
                    EMAIL.NRTIPOORCAMENTO,
                    FILIAL.NMFILIAL";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function validaRegistrosDuplicados($params)
    {

        $sql = "SELECT COUNT (*) AS QTD 
                FROM FCT_PARAMEMAILFILIAL
                WHERE CDFILIAL      = :CDFILIAL
                AND NRPARAMEMAILFILIAL <> NVL(:NRPARAMEMAILFILIAL, -1)";

        return $this->connection->executeQuery($sql, $params)->fetchAllAssociative();
    }

    public function validaRegistrosComEscopoDuplicados($params)
    {

        $sql = "SELECT COUNT(*) AS QTD 
                FROM FCT_PARAMEMAILFILIAL
                WHERE
                    (CDFILIAL = :CDFILIAL OR 
                    (CDFILIAL IS NULL AND :CDFILIAL IS NULL))
                    AND (NRTIPOORCAMENTO = :NRTIPOORCAMENTO OR 
                    (NRTIPOORCAMENTO IS NULL AND :NRTIPOORCAMENTO IS NULL))
                    AND NRPARAMEMAILFILIAL <> NVL(:NRPARAMEMAILFILIAL, -1)";

        return $this->connection->executeQuery($sql, $params)->fetchAllAssociative();
    }

    public function updateParameterEmail($params)
    {
        $sql = "UPDATE FCT_PARAMEMAILFILIAL
                   SET DSPARAMEMAILFILIAL   = :DSPARAMEMAILFILIAL,
                       CDFILIAL             = :CDFILIAL,
                       DSREMETENTEEMAIL     = :DSREMETENTEEMAIL,
                       DSASSUNTOEMAIL       = :DSASSUNTOEMAIL,
                       DSCORPOEMAIL         = :DSCORPOEMAIL,
                       NRTIPOORCAMENTO      = :NRTIPOORCAMENTO,
                       NRORG                = :NRORG,
                       DTULTATU             = SYSDATE,
                       NRORGULTATU          = :NRORGULTATU,
                       CDOPERULTATU         = :CDOPERULTATU,
                       IDATIVO              = :IDATIVO
                 WHERE NRPARAMEMAILFILIAL = :NRPARAMEMAILFILIAL";

        $this->connection->executeStatement($sql, $params);
    }

    public function insertParameterEmail($params)
    {
        $sql = "INSERT INTO FCT_PARAMEMAILFILIAL(
                    NRPARAMEMAILFILIAL, DSPARAMEMAILFILIAL, CDFILIAL, DSREMETENTEEMAIL, DSASSUNTOEMAIL, DSCORPOEMAIL,
                    NRTIPOORCAMENTO, NRORG, NRORGINCLUSAO, DTINCLUSAO, CDOPERINCLUSAO, IDATIVO)
                VALUES(
                    :NRPARAMEMAILFILIAL, :DSPARAMEMAILFILIAL, :CDFILIAL, :DSREMETENTEEMAIL, :DSASSUNTOEMAIL, :DSCORPOEMAIL,
                    :NRTIPOORCAMENTO, :NRORG, :NRORGINCLUSAO, SYSDATE, :CDOPERINCLUSAO, :IDATIVO)";

        $this->connection->executeStatement($sql, $params);
    }

    public function deleteParametrosEmail($params)
    {
        $sql = "DELETE FROM FCT_PARAMEMAILFILIAL WHERE NRPARAMEMAILFILIAL = :NRPARAMEMAILFILIAL";
        $this->connection->executeStatement($sql, $params);
    }
}
