<?php

namespace SM\Service;

use SM\Factory\FmRelatoriosOperacionalContratoFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;

class FmRelatoriosOperacionalContratoService
{
    protected $connection;
    protected $FmRelatoriosOperacionalContratoFactory;
    protected $Environment;
    protected $newCode;

    public function __construct(EntityManager $entityManager, Environment $Environment, NewCode $newCode, FmRelatoriosOperacionalContratoFactory $FmRelatoriosOperacionalContratoFactory)
    {
        $this->connection = $entityManager->getConnection();
        $this->FmRelatoriosOperacionalContratoFactory = $FmRelatoriosOperacionalContratoFactory;
        $this->Environment = $Environment;
        $this->newCode = $newCode;
    }

    public function getRelatoriosTarefas()
    {

        $cdoperador = $this->Environment->getCdOperador();
        $listaOperadores = $this->montarHierarquiaOperadores($cdoperador);

        return $this->FmRelatoriosOperacionalContratoFactory->getRelatoriosTarefas(array(
            "CDOPERADOR" => $cdoperador
        ), $listaOperadores);
    }

    public function getItensTarefas($parametros)
    {
        return $this->FmRelatoriosOperacionalContratoFactory->getItensTarefas($parametros['NRCHECKLISTAGEND']);
    }

    public function getFotosTarefas($parametros)
    {
        return $this->FmRelatoriosOperacionalContratoFactory->getFotosTarefas($parametros['NRCHECKLISTAGEND']);
    }

    public function getFiltroRelatorioLocal()
    {
        $listaLocais = $this->getLocaisHierarquia($this->Environment->getCdOperador());
        return $this->FmRelatoriosOperacionalContratoFactory->getFiltroRelatorioLocal($listaLocais);
    }

    public function getFiltroRelatorioCliente()
    {
        return $this->FmRelatoriosOperacionalContratoFactory->getFiltroRelatorioCliente();
    }
    /* Fim TAB relatórios tarefas */

    /* Estrutura da Hierarquia do Facilities */
    public function getLocaisHierarquia($cdOperador, $entityManager = null)
    {
        $listaLocais = null;

        $locais = $this->FmRelatoriosOperacionalContratoFactory->getLocaisHierarquia(array(
            "CDOPERADOR" => $cdOperador
        ), $entityManager = null);

        foreach ($locais as $local) {
            if ($listaLocais != null) {
                $listaLocais = $listaLocais . ",";
            }
            $listaLocais = $listaLocais . "'" . $local['NRLOCALCLIENTE'] . "'";
        }

        return $listaLocais;
    }

    public function montarHierarquiaOperadores($cdOperador)
    {
        $listaOperadores = null;
        $locais = $this->getLocaisHierarquia($cdOperador);

        $operadores = $this->FmRelatoriosOperacionalContratoFactory->getHierarquiaFacilities(array(
            "CDOPERADOR" => $cdOperador
        ), $locais);

        foreach ($operadores as $operador) {
            if ($listaOperadores != null) {
                $listaOperadores = $listaOperadores . ",";
            }
            $listaOperadores = $listaOperadores . "'" . $operador['CDOPERADOR'] . "'";
        }

        return empty($listaOperadores) ? $cdOperador : $listaOperadores;
    }

    public function findOperadorHierarq($filtro)
    {
        $where = $this->montarFiltro($filtro);
        $listaLocais = $this->getLocaisHierarquia($this->Environment->getCdOperador());
        return $this->FmRelatoriosOperacionalContratoFactory->findOperadorHierarq($where, $listaLocais);
    }

    public function montarFiltro($find)
    {
        $operador = $this->Environment->getCdOperador();

        if (empty($find)) {
            return ' 1 = 1';
        }
        $whereClause = [];
        foreach ($find as $column => $condition) {
            $operator = key($condition);
            $value = $condition[$operator];

            if ($value === 'OPERADOR_LOGADO') {
                $value = $operador;
            }

            if (in_array($operator, ["IN", "NOT_IN"])) {
                $value = "($value)";
            } else {
                $value = "'$value'";
            }

            $whereClause[] = "$column $operator $value";
        }

        return " " . implode(" AND ", $whereClause);
    }

    /* Inicio TAB relatórios Vistorias */
    public function getRelatoriosVistoria()
    {

        $cdoperador = $this->Environment->getCdOperador();
        $listaOperadores = $this->montarHierarquiaOperadores($cdoperador);

        $params = array(
            "CDOPERADOR" => $cdoperador,
        );

        return $this->FmRelatoriosOperacionalContratoFactory->getRelatoriosVistoria($listaOperadores, $params);
    }

    public function getFiltroVistoriaSituacao()
    {
        return $this->FmRelatoriosOperacionalContratoFactory->getFiltroVistoriaSituacao();
    }

    public function getCriteriosVistoria($parametros)
    {
        return $this->FmRelatoriosOperacionalContratoFactory->getCriteriosVistoria($parametros['NRGERENCIAMENTOCHK']);
    }

    public function getFotosVistoria($parametros)
    {
        // Obtenha as fotos usando a função da fábrica
        $fotos = $this->FmRelatoriosOperacionalContratoFactory->getFotosVistoria($parametros['NRGERENCIAMENTOCHK']);

        // Verifique se o resultado está vazio e retorne uma resposta vazia ou padrão
        if (empty($fotos)) {
            return [
                'data' => [],  // Retorna um array vazio, significando que não há fotos
                'message' => 'Nenhuma foto encontrada'  // Mensagem indicando que não há fotos
            ];
        }

        // Se houver fotos, retorne os dados normalmente
        return [
            'data' => $fotos,  // Retorna as fotos obtidas da consulta
            'message' => 'Fotos carregadas com sucesso'
        ];
    }
    /* Fim TAB relatórios Vistorias */

    /* Inicio TAB relatórios auditorias */

    public function getTarefasAuditoria()
    {
        $cdoperador = $this->Environment->getCdOperador();
        $listaOperadores = $this->montarHierarquiaOperadores($cdoperador);
        
        $params = array(
            "CDOPERADOR" => $cdoperador
        );

        return $this->FmRelatoriosOperacionalContratoFactory->getTarefasAuditoria($listaOperadores, $params);
    }

    public function getCategoriasAuditoriaRelatorio($parametros)
    {
        try {
            $categorias = $this->FmRelatoriosOperacionalContratoFactory->getCategoriasAuditoriaRelatorio(array(
                "NRAUDITORIA"   => $parametros['NRAUDITORIA']
            ));

            $categoriasPercentual = $this->getPercentualCategoria($categorias, $parametros);

            return $this->getNotaCategoria($categoriasPercentual, $parametros);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getPercentualCategoria($categorias, $params)
    {
        try {
            $parametrizacao = $this->getParametrizacao();
    
            if ($parametrizacao[0]["IDUTILIZAPERCENTUALAUDIT"] !== 'S') {
                foreach ($categorias['data'] as &$categoria) {
                    $categoria['PERCENTUAL'] = 'N/A';
                }
    
                return $categorias;
            }
    
            $CriteriosSomaPercentual = $this->FmRelatoriosOperacionalContratoFactory->getCriteriosAvaliacaoSomaPercentual();
            $criteriosValidos = array_column($CriteriosSomaPercentual, 'NRCRITERIOAUDIT');
            $percentuaisPorCategoria = [];
    
            foreach ($categorias['data'] as &$categoria) {
                $topicos = $this->getTopicosAuditoriaExecRelatorio(array(
                    'NRAUDITORIA' => $params['NRAUDITORIA'] ?? null,
                    'NRCATEGORIAAUDIT' => $categoria['NRCATEGORIAAUDIT'] ?? null,
                ));
    
                if (!is_array($topicos) || !isset($topicos['data']) || !is_array($topicos['data'])) {
                    $topicos = ['data' => []];
                }
    
                $criteriosSomaPercentualTotal = 0;
                $criteriosSomaPercentualMarcados = 0;
    
                foreach ($topicos['data'] as $topico) {
                    $criterios = $this->FmRelatoriosOperacionalContratoFactory->getCriteriosAudit(array(
                        'NRTOPICOAUDIT' => $topico['NRTOPICOAUDIT'] ?? null,
                        'NRAUDITORIA'   => $params['NRAUDITORIA'] ?? null,
                    ));
    
                    foreach ($criterios as $criterio) {
                        if (in_array($criterio['NRCRITERIOAUDIT'], $criteriosValidos)) {
                            $criteriosSomaPercentualTotal++;
                            if ($criterio['IDREALIZADO'] == 'S') {
                                $criteriosSomaPercentualMarcados++;
                            }
                        }
                    }
                }
    
                if ($criteriosSomaPercentualTotal > 0) {
                    $percentualConclusao = ($criteriosSomaPercentualMarcados / $criteriosSomaPercentualTotal) * 100;
                    $percentuaisPorCategoria = round($percentualConclusao, 0);
                } else {
                    $percentuaisPorCategoria = 0;
                }
    
                $categoria['PERCENTUAL'] = $percentuaisPorCategoria;
            }
    
            return $categorias;
        } catch (\Exception $e) {
            error_log("Erro em getPercentualCategoria: " . $e->getMessage());
            throw $e;
        }
    }

    public function getParametrizacao() 
    {
        try {
            return $this->FmRelatoriosOperacionalContratoFactory->getParametrizacao();
        } catch (\Exception $e) {
            throw $e;
        } 
    }

    public function getTopicosAuditoriaExecRelatorio($parametros)
    {
        return $this->FmRelatoriosOperacionalContratoFactory->getTopicosAuditoriaExecRelatorio(array(
            "NRAUDITORIA"        => $parametros['NRAUDITORIA'],
            "NRCATEGORIAAUDIT"   => $parametros['NRCATEGORIAAUDIT'],
        ));
    }

    public function getNotaCategoria($categorias, $params)
    {
        try {
            $parametrizacao = $this->getParametrizacao();
    
            if ($parametrizacao[0]['IDUTILIZANOTAAUDIT'] === 'S') {
                $CriteriosSomaNota = $this->FmRelatoriosOperacionalContratoFactory->getCriteriosAvaliacaoSomaNota();
                $criteriosValidos = array_column($CriteriosSomaNota, 'NRCRITERIOAUDIT');
    
                foreach ($categorias['data'] as &$categoria) {
                    $resultados = [];
                    
                    $topicos = $this->getTopicosAuditoriaExecRelatorio(array(
                        'NRAUDITORIA' => $params['NRAUDITORIA'] ?? null,
                        'NRCATEGORIAAUDIT' => $categoria['NRCATEGORIAAUDIT'] ?? null,
                    ));
    
                    // Validação do retorno de $topicos
                    if (!is_array($topicos) || !isset($topicos['data']) || !is_array($topicos['data'])) {
                        $topicos = ['data' => []];
                    }
    
                    foreach ($topicos['data'] as $topico) { // Linha 1044 ajustada
                        $arryCrit = array(
                            'NRCATEGORIAAUDIT' => $categoria['NRCATEGORIAAUDIT'] ?? null,
                            'NRTOPICOAUDIT' => $topico['NRTOPICOAUDIT'] ?? null,
                            'NRAUDITORIA'   => $params['NRAUDITORIA'] ?? null,
                        );
                        
                        $criterios = $this->FmRelatoriosOperacionalContratoFactory->getCriteriosMarcadosAudit($arryCrit);
                        
                        foreach ($criterios as $criterio) {
                            if (in_array($criterio['NRCRITERIOAUDIT'], $criteriosValidos)) {
                                $resultados[] = array(
                                    'NRCATEGORIAAUDIT' => $categoria['NRCATEGORIAAUDIT'],
                                    'NRTOPICOAUDIT'    => $topico['NRTOPICOAUDIT'],
                                    'NRCRITERIOAUDIT'  => $criterio['NRCRITERIOAUDIT'],
                                );
                            }
                        }
                    }
    
                    $notaTotalCategoria = 0;
    
                    foreach ($resultados as $resultado) {
                        $notaTopico = $this->FmRelatoriosOperacionalContratoFactory->getNotaTopico($resultado);
    
                        if (!empty($notaTopico)) {
                            $notaTotalCategoria += $notaTopico[0]['NOTATOPICAUDIT'];
                        }
                    }
                    $categoria['NOTA'] = round($notaTotalCategoria, 2);
                }
    
                return $categorias;
            } else {
                foreach ($categorias['data'] as &$categoria) {
                    $categoria['NOTA'] = 'N/A';
                }
    
                return $categorias;
            }
        } catch (\Exception $e) {
            error_log("Erro em getNotaCategoria: " . $e->getMessage());
            throw $e;
        }
    }

    public function getCriteriosAuditoriaExecRelatorio($parametros)
    {
        return $this->FmRelatoriosOperacionalContratoFactory->getCriteriosAuditoriaExecRelatorio(array(
            "NRCATEGORIAAUDIT"  => $parametros['NRCATEGORIAAUDIT'],
            "NRAUDITORIA"       => $parametros['NRAUDITORIA'],
            "NRTOPICOAUDIT"     => $parametros['NRTOPICOAUDIT'],
        ));
    }

    public function getImagensTopicosAuditoria($parametros)
    {
        return $this->FmRelatoriosOperacionalContratoFactory->getImagensTopicosAuditoria(array(
            "NRAUDITORIA"        => $parametros['NRAUDITORIA'],
            "NRCATEGORIAAUDIT"  => $parametros['NRCATEGORIAAUDIT'],
            "NRTOPICOAUDIT"     => $parametros['NRTOPICOAUDIT'],
        ));
    }
    /* Fim TAB relatórios auditorias */
}