<?php

namespace SM\Service;

use SM\Factory\ProposalModelFactory;
use SM\Service\FindService;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use SM\Service\FileServerService;
use SM\Util\Utilities;
use Doctrine\ORM\EntityManager;

class ProposalModelService
{
    protected $ProposalModelFactory;
    protected $findService;
    protected $Environment;
    protected $newCode;
    protected $connection;

    public function __construct(EntityManager $entityManager, ProposalModelFactory $ProposalModelFactory, FindService $findService, Environment $Environment, NewCode $newCode)
    {
        $this->connection = $entityManager->getConnection();
        $this->ProposalModelFactory = $ProposalModelFactory;
        $this->findService = $findService;
        $this->Environment = $Environment;
        $this->newCode = $newCode;
    }

    public function getProposalModel()
    {
        return $this->ProposalModelFactory->getProposalModel();
    }

    public function getTabEscopoFaturamentoProposal($params)
    {
        $params = array(
            "NRORCAMENTOMODELO" => $params['NRORCAMENTOMODELO']
        );
        return $this->ProposalModelFactory->getTabEscopoFaturamentoProposal($params);
    }

    public function getEscopoFaturamentoModeloDoc($params)
    {
        $params = array(
            "NRRELACESCOPOMODELO" => $params['NRRELACESCOPOMODELO']
        );
        return $this->ProposalModelFactory->getEscopoFaturamentoModeloDoc($params);
    }

    /**
     * Valida se o usuário está tentando inserir uma situação existente
     **/
    public function validarExistenciaIdsituacao($row, $nrorg, $idSituacao)
    {
        try {

            $params = array(
                "NRRELACESCOPOMODELO" => $row["NRRELACESCOPOMODELO"],
                "NRSEQMODDOCFCT" => $row["NRSEQMODDOCFCT"],
                "NRORG" => $nrorg
            );

            $existe = $this->ProposalModelFactory->getModeloDocumentoSituacao($idSituacao, $params);

            if (count($existe) > 0 && $existe[0]["IDSITUACAOORCAMENTO"]) {
                $situacoes = "SITUACAO_JA_CADASTRADA::" . $existe[0]["IDSITUACAOORCAMENTO"];
                $situacoes = str_replace('ELABORACAO', 'Em Elaboração', $situacoes);
                $situacoes = str_replace('CONCLUIDO', 'Concluída', $situacoes);
                $situacoes = str_replace('ACEITO', 'Aceita', $situacoes);
                $situacoes = str_replace('CANCELADO', 'Cancelada', $situacoes);
                $situacoes = str_replace('ENVIADO', 'Documento Enviado', $situacoes);
                throw new \Exception($situacoes);
            }

        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function saveEscopoFaturamentoModeloDoc($row)
    {
        $this->connection->beginTransaction();

        try {

            $nrorg = $this->Environment->getNrOrg();
            $cdoperador = $this->Environment->getCdOperador();

            if ($row['NRORCAESCOPODOC'] == null) {

                foreach ($row["IDSITUACAOORCAMENTOEDIT"] as $idSituacao) {
                    $this->validarExistenciaIdsituacao($row, $nrorg, $idSituacao);
                    $nrorcaescopodoc = $this->newCode->getCodeByTableName("FCT_ORCAESCOPODOC", $nrorg, 1, 10, true);
                    $params = array(
                        "NRORCAESCOPODOC" => $nrorcaescopodoc,
                        "NRRELACESCOPOMODELO" => $row["NRRELACESCOPOMODELO"],
                        "NRSEQMODDOCFCT" => $row["NRSEQMODDOCFCT"],
                        "IDPERMITEDOCX" => $row["IDPERMITEDOCX"],
                        "IDATIVO" => 'S',
                        "NRORG" => $nrorg,
                        "IDSITUACAOORCAMENTO" => $idSituacao,
                        "CDOPERADOR" => $cdoperador
                    );

                    $this->ProposalModelFactory->inserirDocumentoSituacao($params);
                }
            } else {

                $idsitadicionadas = array_diff($row["IDSITUACAOORCAMENTOEDIT"], $row["IDSITUACAOORCAMENTO"]);
                $idsitremovidas = array_diff($row["IDSITUACAOORCAMENTO"], $row["IDSITUACAOORCAMENTOEDIT"]);

                if (count($idsitadicionadas) > 0) {

                    foreach ($idsitadicionadas as $idSituacao) {
                        $this->validarExistenciaIdsituacao($row, $nrorg, $idSituacao);
                        $nrorcaescopodoc = $this->newCode->getCodeByTableName("FCT_ORCAESCOPODOC", $nrorg, 1, 10, true);
                        $params = array(
                            "NRORCAESCOPODOC" => $nrorcaescopodoc,
                            "NRRELACESCOPOMODELO" => $row["NRRELACESCOPOMODELO"],
                            "NRSEQMODDOCFCT" => $row["NRSEQMODDOCFCT"],
                            "IDPERMITEDOCX" => $row["IDPERMITEDOCX"],
                            "IDATIVO" => 'S',
                            "NRORG" => $nrorg,
                            "IDSITUACAOORCAMENTO" => $idSituacao,
                            "CDOPERADOR" => $cdoperador
                        );

                        $this->ProposalModelFactory->inserirDocumentoSituacao($params);
                    }
                }

                if (count($idsitremovidas) > 0) {

                    foreach ($idsitremovidas as $idSituacao) {

                        $params = array(
                            "NRRELACESCOPOMODELO" => $row["NRRELACESCOPOMODELO"],
                            "NRORG" => $nrorg,
                        );

                        $this->ProposalModelFactory->excluirDocumentoSituacao($idSituacao, $params);
                    }
                }

                if ((count($idsitremovidas) == 0) && (count($idsitadicionadas) == 0)) {
                    $params = array(
                        "NRSEQMODDOCFCT" => $row["NRSEQMODDOCFCT"],
                        "NRRELACESCOPOMODELO" => $row["NRRELACESCOPOMODELO"],
                        "IDPERMITEDOCX" => $row["IDPERMITEDOCX"],
                        "NRORG" => $nrorg,
                        "CDOPERADOR" => $cdoperador
                    );

                    $this->ProposalModelFactory->editarDocumentoSituacao($params);
                }
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function getTabComposicoes($params)
    {
        $params = array(
            "NRORCAMENTOMODELO" => $params['NRORCAMENTOMODELO']
        );

        return $this->ProposalModelFactory->getTabComposicoes($params);
    }

    public function getTabItens($params)
    {
        $params = array(
            "NRORCAMENTOMODELO" => $params['NRORCAMENTOMODELO']
        );

        return $this->ProposalModelFactory->getTabItens($params);
    }

    public function getTabPerfisAcessoProposal($params)
    {
        $params = array(
            "NRORCAMENTOMODELO" => $params['NRORCAMENTOMODELO']
        );
        return $this->ProposalModelFactory->getTabPerfisAcessoProposal($params);
    }

    public function getTabSignatariosPadroesProposal($params)
    {
        $params = array(
            "NRORCAMENTOMODELO" => $params['NRORCAMENTOMODELO']
        );
        return $this->ProposalModelFactory->getTabSignatariosPadroesProposal($params);
    }

    public function getServicoProduto()
    {
        return $this->ProposalModelFactory->getServicoProduto();
    }

    public function getTipoMedidaProposal()
    {
        return $this->ProposalModelFactory->getTipoMedidaProposal();
    }

    public function saveModalAddProposalModel($params)
    {
        $this->connection->beginTransaction();

        try {

            $nrorg = $this->Environment->getNrOrg();
            $cdoperador = $this->Environment->getCdOperador();
            $nrorcamentomodelo = $this->newCode->getCodeByTableName("FCT_ORCAMENTOMODELO", $nrorg, 1, 10, true);
            $dataParams = array(
                "NRORCAMENTOMODELO" => $nrorcamentomodelo,
                "IDATIVO" => $params["IDATIVO"],
                "IDBLOQALTERADADOS" => $params["IDBLOQALTERADADOS"],
                "IDFATURACONTR" => "A",
                "NMORCAMENTOMODELO" => $params["NMORCAMENTOMODELO"],
                "QTDIASVALIDADE" => $params["QTDIASVALIDADE"],
                "QTDIASCONTRATO" => $params["QTDIASCONTRATO"],
                "IDUTILIZACOMP" => $params["IDUTILIZACOMP"],
                "NRDIASCARENCIA" => $params["NRDIASCARENCIA"],
                "IDOBRIGAOPORTUNIDADE" => $params["IDOBRIGAOPORTUNIDADE"],
                "NRSEQMODCONTRA" => $params["NRSEQMODCONTRA"],
                "CDMINEMORCA" => $params["CDMINEMORCA"],
                "IDUTILIZAREVENDA" => $params["IDUTILIZAREVENDA"],
                "DSREFERENCIAORCA" => $params["DSREFERENCIAORCA"],
                "DSOBSERVACAOORCA" => $params["DSOBSERVACAOORCA"],
                "IDASSELETRDIGITAL" => $params["IDASSELETRDIGITAL"],
                "IDENVIADOCCONCLUSAO" => $params["IDENVIADOCCONCLUSAO"],
                "NRTIPODOCCERTSIGN" => $params["NRTIPODOCCERTSIGN"],
                "IDENVIADOCOPERRESPCONCASS" => $params["IDENVIADOCOPERRESPCONCASS"],
                "NRORG" => $nrorg,
                "CDOPERINCLUSAO" => $cdoperador,
            );

            $this->ProposalModelFactory->insereSignatariosPadroesOrca($dataParams);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    /**
     * insere os signatários padrões de modelo de proposta que foram buscados na parametrização SM 
     */
    // public function saveModalAddProposalModelTabSignatariosPadroes($parametros)
    // {
    //     try {
    //         Obter NRORG do ambiente
    //         $nrorg = $this->Environment->getNrOrg();

    //         Validar token certsign
    //         $certsign = $this->ProposalModelFactory->validaTokenCertsign(["NRORG" => $nrorg]);

    //         if (empty($certsign)) {
    //             throw new \Exception("Token Certsign inválido ou não encontrado.");
    //         }

    //         Buscar signatários padrões
    //         $signatarios = $this->ProposalModelFactory->buscaSignatariosPadroes(["NRORG" => $nrorg]);

    //         if (empty($signatarios)) {
    //             throw new \Exception("Nenhum signatário padrão encontrado para o NRORG informado.");
    //         }

    //         foreach ($signatarios as $signatario) {
    //             $paramsValidacao = [
    //                 "NRORCAMENTOMODELO" => $parametros["NRORCAMENTOMODELO"] ?? null,
    //                 "NRPARCNEGOCIO"     => $signatario["NRPARCNEGOCIO"]?? null,
    //                 "IDTPSIGNATARIO"    => $signatario["IDTPSIGNATARIO"]?? null,
    //                 "NRORG"             => $nrorg,
    //             ];

    //             $valida = $this->ProposalModelFactory->validaSignatCadastrados($paramsValidacao);

    //             if ($valida == 0) {
    //                 Gerar novo código para NRSIGNATPADRORCA
    //                 $nrsignatpadrorca = $this->newCode->getCodeByTableName("FCT_SIGNATPADRMODORCA", $nrorg, 1, 10, true);

    //                 $paramsInsercao = [
    //                     "NRSIGNATPADRORCA"   => $nrsignatpadrorca,
    //                     "NRORCAMENTOMODELO"  => $parametros["NRORCAMENTOMODELO"]?? 0,
    //                     "NRPARCNEGOCIO"      => $signatario["NRPARCNEGOCIO"],
    //                     "IDTIPOASSINATURA"   => $signatario["IDTIPOASSINATURA"],
    //                     "IDTESTEMUNHA"       => $signatario["IDTESTEMUNHA"],
    //                     "NRORDENACAO"        => $signatario["NRORDENACAO"],
    //                     "IDTPSIGNATARIO"     => $signatario["IDTPSIGNATARIO"],
    //                     "DSEMAILNOTSIGNENV"  => $signatario["DSEMAILNOTSIGNENV"], 
    //                     "DSEMAILNOTIFOUTROS" => $signatario["DSEMAILNOTIFOUTROS"],
    //                     "NRORG"              => $nrorg,
    //                     "OPERADOR"           => $parametros["CDOPERINCLUSAO"],
    //                 ];

    //                 $this->ProposalModelFactory->insereSignatariosPadroes($paramsInsercao);

    //                 Buscar operadores relacionados ao signatário
    //                 $operadores = $this->ProposalModelFactory->buscaOperadoresNotif([
    //                     "NRSIGNATARIOSPADR" => $signatario["NRSIGNATARIOSPADR"],
    //                     "NRORG"             => $nrorg,
    //                 ]);

    //                 foreach ($operadores as $operador) {
    //                     $nrrelopernotifassele = $this->newCode->getCodeByTableName("FCT_RELACOPERNOTIFASSELE", $nrorg, 1, 10, true);

    //                     $paramsOperador = [
    //                         "NRRELOPERNOTIFASSELE" => $nrrelopernotifassele,
    //                         "NRSIGNATPADRORCA"     => $nrsignatpadrorca,
    //                         "CDOPERADOR"           => $operador["CDOPERADOR"],
    //                         "NRORG"                => $nrorg,
    //                         "CDOPERAINCLUSAO"      => $parametros["CDOPERINCLUSAO"],
    //                     ];

    //                     $this->ProposalModelFactory->insereOperadoresNotifica($paramsOperador);
    //                 }
    //             }
    //         }
    //     } catch (\Exception $e) {
    //         Log do erro
    //         error_log("Erro ao inserir signatários padrão: " . $e->getMessage());
    //         throw $e;
    //     }
    // }
    public function saveModalAddProposalModelTabSignatariosPadroes($parametros)
    {
        try {
            $nrorg = $this->Environment->getNrOrg();
            $cdoperador = $this->Environment->getCdOperador();
            $nrsignatpadrorca = $this->newCode->getCodeByTableName("FCT_SIGNATPADRMODORCA", $nrorg, 1, 10, true);

            $params = [
                "NRSIGNATPADRORCA" => $nrsignatpadrorca,
                "NRORCAMENTOMODELO" => $parametros["NRORCAMENTOMODELO"],
                "NRPARCNEGOCIO" => $parametros["NRPARCNEGOCIO"],
                "IDTIPOASSINATURA" => $parametros["IDTIPOASSINATURA"],
                "IDTESTEMUNHA" => $parametros["IDTESTEMUNHA"],
                "NRORDENACAO" => $parametros["NRORDENACAO"],
                "IDTPSIGNATARIO" => $parametros["IDTPSIGNATARIO"],
                // "DSEMAILNOTSIGNENV"  => $parametros["DSEMAILNOTSIGNENV"],
                // "DSEMAILNOTIFOUTROS" => $parametros["DSEMAILNOTIFOUTROS"],
                "NRORG" => $nrorg,
                "OPERADOR" => $cdoperador,
            ];

            $this->ProposalModelFactory->insereSignatariosPadroes($params);
        } catch (\Exception $e) {
            error_log("Erro ao inserir signatário padrão: " . $e->getMessage());
            throw $e;
        }
    }




    public function deleteProposalModel($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $retorno = $this->findService->formatarArray($parametros["NRS"]);
            $this->ProposalModelFactory->deleteProposalModel($retorno);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deleteProposalModelTabEscopoFaturamento($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $retorno = $this->findService->formatarArray($parametros["NRS"]);
            $this->ProposalModelFactory->deleteProposalModelTabEscopoFaturamento($retorno);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deleteProposalModelTabitensDocumento($parametros)
    {
        $this->connection->beginTransaction();

        try {

            foreach ($parametros["NRS"] as $item) {

                $params = array(
                    "NRITEMORCAMODELO" => $item["NRITEMORCAMODELO"],
                );

                $this->ProposalModelFactory->deleteProposalModelitens($params);
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deleteProposalModelTabEscFatModeloDocumento($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->Environment->getNrOrg();

            foreach ($parametros["NRS"] as $item) {

                $params = array(
                    "NRRELACESCOPOMODELO" => $item["NRRELACESCOPOMODELO"],
                    "NRSEQMODDOCFCT" => $item["NRSEQMODDOCFCT"],
                    "NRORG" => $nrorg,
                );

                $this->ProposalModelFactory->excluirEscFatModeloDocumento($params);
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deleteProposalModelTabComposicoes($parametros)
    {
        $this->connection->beginTransaction();

        try {
            if (isset($parametros["NRS"])) {
                $retorno = $this->findService->formatarArray($parametros["NRS"]);
                $retorno = str_replace("'", "", $retorno);
            } else {
                $retorno = $parametros;
            }

            $this->ProposalModelFactory->deleteProposalModelTabComposicoes($retorno);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deleteProposalModelTabPerfisAcesso($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $retorno = $this->findService->formatarArray($parametros["NRS"]);
            $this->ProposalModelFactory->deleteProposalModelTabPerfisAcesso($retorno);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deleteProposalModelTabSignatariosPadroes($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $retorno = $this->findService->formatarArray($parametros["NRSIGNATPADRORCA"]);
            $this->ProposalModelFactory->deleteProposalModelTabSignatariosPadroes($retorno);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function getEscopoDeFaturamentoProposal()
    {
        return $this->ProposalModelFactory->getEscopoDeFaturamentoProposal();
    }

    public function getModeloContratoProposal()
    {
        return $this->ProposalModelFactory->getModeloContratoProposal();
    }

    /**
     * A função "validaEscopoProposta" verifica se o escopo 
     * que está sendo relacionado já está relacionado ao modelo.
     */

    public function validaEscopoProposta($tipoOrcamento, $orcaModelo)
    {

        $paramsValidaEscopoProp = array(
            "NRTIPOORCAMENTO" => $tipoOrcamento,
            "NRORCAMENTOMODELO" => $orcaModelo,
            "NRORG" => $this->Environment->getNrOrg()
        );

        $returnValidaEscopoProp = $this->ProposalModelFactory->validaEscopoProposta($paramsValidaEscopoProp);

        return $returnValidaEscopoProp[0]["VALIDAESCOPO"] != 0;

    }

    public function saveModalAddProposalModelTabEscopoFaturamento($params)
    {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->Environment->getNrOrg();
            $cdoperador = $this->Environment->getCdOperador();
            $orcaModelo = $params["NRORCAMENTOMODELO"];

            foreach ($params["NRTIPOORCAMENTO"] as $tipoOrcamento) {
                $existe = $this->validaEscopoProposta($tipoOrcamento, $orcaModelo);

                if (!$existe) {
                    $nrrelacescopomodelo = $this->newCode->getCodeByTableName("FCT_RELACESCOPOMODELO", $nrorg, 1, 10, true);

                    $this->ProposalModelFactory->saveModalAddProposalModelTabEscopoFaturamento(array(
                        "NRORCAMENTOMODELO" => $params["NRORCAMENTOMODELO"],
                        "NRTIPOORCAMENTO" => $tipoOrcamento,
                        "IDVERIFICADOC" => $params["IDVERIFICADOC"],
                        "NRRELACESCOPOMODELO" => $nrrelacescopomodelo,
                        "IDATIVO" => "S",
                        "NRORG" => $nrorg,
                        "CDOPERINCLUSAO" => $cdoperador,
                    ));
                } else {
                    throw new \Exception("EXISTE!");
                }
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function saveModalAddProposalModelTabComposicoes($params)
    {
        $this->connection->beginTransaction();
        try {
            $CDOPERADOR = $this->Environment->getCdOperador();
            $NRORG = $this->Environment->getNrOrg();
            $NRRELORCAMODCOMP = $this->newCode->getCodeByTableName("FCT_RELORCAMODCOMPOSICAO", $NRORG, 1, 10, true);

            $this->ProposalModelFactory->saveModalAddProposalModelTabComposicoes(array(
                "NRRELORCAMODCOMP" => $NRRELORCAMODCOMP,
                "NRORCAMENTOMODELO" => $params["NRORCAMENTOMODELO"],
                "NRCOMPSERVICO" => $params["NRCOMPSERVICO"],
                "IDATIVO" => "S",
                "NRORG" => $NRORG,
                "CDOPERINCLUSAO" => $CDOPERADOR,
                "CDOPERULTATU" => $CDOPERADOR,
            ));

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function saveModalAddProposalModelTabPerfisAcesso($params)
    {
        $this->connection->beginTransaction();

        try {
            $NRORG = $this->Environment->getNrOrg();
            $NRRELORCAMODPERFACESS = $this->newCode->getCodeByTableName("FCT_RELORCAMODPERFACESS", $NRORG, 1, 10, true);

            // Ajuste na lógica de atribuição de NRORCAMENTOMODELO
            $NRPERFILACESSO = isset($params["NRPERFILACESSO"]) && !empty($params["NRPERFILACESSO"])
                ? $params["NRPERFILACESSO"]
                : 0;

            $this->ProposalModelFactory->saveModalAddProposalModelTabPerfisAcesso(array(
                "NRORCAMENTOMODELO" => isset($params["NRORCAMENTOMODELO"]),
                "IDATIVO" => "S",
                "NRPERFILACESSO" => $NRPERFILACESSO,
                "NRORG" => $NRORG,
                "CDOPERINCLUSAO" => $this->Environment->getCdOperador(),
                "NRRELORCAMODPERFACESS" => $NRRELORCAMODPERFACESS,
            ));
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function editProposalModelTabPerfisAcesso($params)
    {
        $this->connection->beginTransaction();
    
        try {
            $nrorg = $this->Environment->getNrorg();
            $operador = $this->Environment->getCdOperador();
            if (isset($params["DRG"]) && is_array($params["DRG"])) {
                foreach ($params["DRG"] as $row) {
                    if (is_array($row) && isset($row["NRRELORCAMODPERFACESS"])) {
                        if ($row["NRRELORCAMODPERFACESS"] != 0) {
                            $parametros = array(
                                "IDATIVO"               => "S",
                                "NRPERFILACESSO"        => $row["NRPERFILACESSO"],
                                "NRORG"                 => $nrorg,
                                "CDOPERINCLUSAO"        => $this->Environment->getCdOperador(),
                                "NRRELORCAMODPERFACESS" => $row["NRRELORCAMODPERFACESS"],
                                "CDOPERULTATU"          => $operador
                            );
    
                            $this->ProposalModelFactory->updateModalAddProposalModelTabPerfisAcesso($parametros);
                        } else {
                            $NRRELORCAMODPERFACESS = $this->newCode->getCodeByTableName("FCT_RELORCAMODPERFACESS", $nrorg, 1, 10, true);
                            $parametros = array(
                                "IDATIVO"               => "S",
                                "NRORCAMENTOMODELO"     => $params["NRORCAMENTOMODELO"],
                                "NRPERFILACESSO"        => $row["NRPERFILACESSO"],
                                "NRORG"                 => $nrorg,
                                "CDOPERINCLUSAO"        => $this->Environment->getCdOperador(),
                                "NRRELORCAMODPERFACESS" => $NRRELORCAMODPERFACESS,
                            );
    
                            $this->ProposalModelFactory->saveModalAddProposalModelTabPerfisAcesso($parametros);
                        }
                    }
                }
            }
    
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }    

    public function getComposicoes()
    {
        return $this->ProposalModelFactory->getComposicoes();
    }

    public function getTipoDocCertsign()
    {
        return $this->ProposalModelFactory->getTipoDocCertsign();
    }

    public function getPerfisDeAcessoProposal()
    {
        return $this->ProposalModelFactory->getPerfisDeAcessoProposal();
    }

    public function getSignatariosProposalModel()
    {
        return $this->ProposalModelFactory->getSignatariosProposalModel();
    }

    public function getOperadoresProposalModel()
    {
        return $this->ProposalModelFactory->getOperadoresProposalModel();
    }

    public function getCpfSignatario($params)
    {
        $params = array(
            "NRPARCNEGOCIO" => $params['signatario']
        );
        return $this->ProposalModelFactory->getCpfSignatario($params);
    }

    public function editProposalModelViewItens($parametros)
    {
        $this->connection->beginTransaction();
        try {
            $NRORG = $this->Environment->getNrOrg();
            $CDOPERADOR = $this->Environment->getCdOperador();

            if ($parametros["NRITEMORCAMODELO"] != 0) {
                $params = [
                    "NRITEMORCAMODELO" => $parametros["NRITEMORCAMODELO"],
                    "NRORCAMENTOMODELO" => $parametros["NRORCAMENTOMODELO"],
                    "CDSERVORCA" => $parametros["CDSERVORCA"],
                    "NRITEMORCAMODELOPAI" => $parametros["NRITEMORCAMODELOPAI"],
                    "NMITEMORCAMENTO" => $parametros["NMITEMORCAMENTO"],
                    "NMREDUITEMORCA" => $parametros["NMREDUITEMORCA"],
                    "NMMNEMOITEMORCA" => $parametros["NMMNEMOITEMORCA"],
                    "DSQTITEMORCA" => $parametros["DSQTITEMORCA"],
                    "IDIMPRESSAOORCA" => $parametros["IDIMPRESSAOORCA"],
                    "QTMEDIDAORCA" => $parametros["QTMEDIDAORCA"],
                    "VRUNITITEMORCA" => $parametros["VRUNITITEMORCA"],
                    "DSQTITEMORCAADIC" => $parametros["DSQTITEMORCAADIC"],
                    "QTITEMORCAADIC" => $parametros["QTITEMORCAADIC"],
                    "VRTOTALITEMORCA" => $parametros["VRTOTALITEMORCA"],
                    "QTDIASINICFAT" => $parametros["QTDIASINICFAT"],
                    "IDITEMOBRIGORCA" => $parametros["IDITEMOBRIGORCA"],
                    "IDMULTMEDVRTOT" => $parametros["IDMULTMEDVRTOT"],
                    "IDBLOQALTORC" => $parametros["IDBLOQALTORC"],
                    "DTULTATU" => "SYSDATE",
                    "NRORGULTATU" => $NRORG,
                    "CDOPERULTATU" => $CDOPERADOR,
                    "IDITEMESPELHO" => $parametros["IDITEMESPELHO"],
                    "NRTIPOHIERARQSERV" => isset($parametros["NRTIPOHIERARQSERV"]),
                    "QTPARCELASFATU" => isset($parametros["QTPARCELASFATU"]),
                    "NRDIAVENCPARCELASFATU" => isset($parametros["NRDIAVENCPARCELASFATU"]),
                ];
                $this->ProposalModelFactory->updateProposalModelTabItens($params);
            } else {
                $NRITEMORCAMODELO = $this->newCode->getCodeByTableName("FCT_ITEMORCAMODELO", $NRORG, 1, 10, true);
                $params = [
                    "NRITEMORCAMODELO" => $NRITEMORCAMODELO,
                    "NRORCAMENTOMODELO" => $parametros["NRORCAMENTOMODELO"],
                    "CDSERVORCA" => $parametros["CDSERVORCA"],
                    "NRITEMORCAMODELOPAI" => $parametros["NRITEMORCAMODELOPAI"],
                    "NMITEMORCAMENTO" => $parametros["NMITEMORCAMENTO"],
                    "NMREDUITEMORCA" => $parametros["NMREDUITEMORCA"],
                    "NMMNEMOITEMORCA" => $parametros["NMMNEMOITEMORCA"],
                    "DSQTITEMORCA" => $parametros["DSQTITEMORCA"],
                    "IDIMPRESSAOORCA" => $parametros["IDIMPRESSAOORCA"],
                    "QTMEDIDAORCA" => $parametros["QTMEDIDAORCA"],
                    "VRUNITITEMORCA" => $parametros["VRUNITITEMORCA"],
                    "DSQTITEMORCAADIC" => $parametros["DSQTITEMORCAADIC"],
                    "QTITEMORCAADIC" => $parametros["QTITEMORCAADIC"],
                    "VRTOTALITEMORCA" => $parametros["VRTOTALITEMORCA"],
                    "QTDIASINICFAT" => $parametros["QTDIASINICFAT"],
                    "IDITEMOBRIGORCA" => $parametros["IDITEMOBRIGORCA"],
                    "CDTPMEDIDASERV" => $parametros["CDTPMEDIDASERV"],
                    "QTITEMORCAMENTO" => $parametros["QTITEMORCAMENTO"],
                    "IDMULTMEDVRTOT" => $parametros["IDMULTMEDVRTOT"],
                    "IDBLOQALTORC" => $parametros["IDBLOQALTORC"],
                    "NRORG" => $NRORG,
                    "IDATIVO" => "S",
                    "DTINCLUSAO" => "SYSDATE",
                    "NRORGINCLUSAO" => $NRORG,
                    "CDOPERINCLUSAO" => $CDOPERADOR,
                    "NRORGULTATU" => $NRORG,
                    "CDOPERULTATU" => $CDOPERADOR,
                    "IDITEMESPELHO" => $parametros["IDITEMESPELHO"],
                    "NRTIPOHIERARQSERV" => isset($parametros["NRTIPOHIERARQSERV"]),
                    "QTPARCELASFATU" => isset($parametros["QTPARCELASFATU"]),
                    "NRDIAVENCPARCELASFATU" => isset($parametros["NRDIAVENCPARCELASFATU"]),
                ];
                $this->ProposalModelFactory->saveModalAddProposalModelTabItens($params);
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function editProposalModelTabSignatariosPadroes($params)
    {
        $this->connection->beginTransaction();

        try {

            $nrorg = $this->Environment->getNrorg();
            $operador = $this->Environment->getCdOperador();

            $updateParams = [
                'NRSIGNATPADRORCA' => isset($params['NRSIGNATPADRORCA']),
                'NRORCAMENTOMODELO' => $params['NRORCAMENTOMODELO'],
                'NRORDENACAO' => $params['NRORDENACAO'],
                'NRORG' => $nrorg,
                'OPERADOR' => $operador,
                'NRORGULTATU' => $nrorg
            ];

            $this->ProposalModelFactory->updateSignatariosPadroes($updateParams);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }


    public function editProposalModelItens($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->Environment->getNrorg();
            $operador = $this->Environment->getCdOperador();

            $this->ProposalModelFactory->updateProposalModelTabItens(array(
                "NRITEMORCAMODELO" => $parametros["NRITEMORCAMODELO"],
                "NRORCAMENTOMODELO" => $parametros["NRORCAMENTOMODELO"],
                "CDSERVORCA" => $parametros["CDSERVORCA"],
                "NRITEMORCAMODELOPAI" => $parametros["NRITEMORCAMODELOPAI"],
                "NMITEMORCAMENTO" => isset($parametros["NMITEMORCAMENTO"]),
                "NMREDUITEMORCA" => $parametros["NMREDUITEMORCA"],
                "NMMNEMOITEMORCA" => $parametros["NMMNEMOITEMORCA"],
                "DSQTITEMORCA" => $parametros["DSQTITEMORCA"],
                "IDIMPRESSAOORCA" => $parametros["IDIMPRESSAOORCA"],
                "QTMEDIDAORCA" => $parametros["QTMEDIDAORCA"],
                "VRUNITITEMORCA" => $parametros["VRUNITITEMORCA"],
                "DSQTITEMORCAADIC" => $parametros["DSQTITEMORCAADIC"],
                "QTITEMORCAADIC" => $parametros["QTITEMORCAADIC"],
                "VRTOTALITEMORCA" => $parametros["VRTOTALITEMORCA"],
                "QTDIASINICFAT" => $parametros["QTDIASINICFAT"],
                "IDITEMOBRIGORCA" => $parametros["IDITEMOBRIGORCA"],
                "CDTPMEDIDASERV" => $parametros["CDTPMEDIDASERV"],
                "QTITEMORCAMENTO" => $parametros["QTITEMORCAMENTO"],
                "IDMULTMEDVRTOT" => $parametros["IDMULTMEDVRTOT"],
                "IDBLOQALTORC" => $parametros["IDBLOQALTORC"],
                "NRORG" => $nrorg,
                "IDATIVO" => "S",
                "DTINCLUSAO" => "SYSDATE",
                "NRORGINCLUSAO" => $nrorg,
                "CDOPERINCLUSAO" => $operador,
                "NRORGULTATU" => $nrorg,
                "CDOPERULTATU" => $operador,
                "IDITEMESPELHO" => $parametros["IDITEMESPELHO"],
                "NRTIPOHIERARQSERV" => isset($parametros["NRTIPOHIERARQSERV"]),
                "QTPARCELASFATU" => isset($parametros["QTPARCELASFATU"]),
                "NRDIAVENCPARCELASFATU" => isset($parametros["NRDIAVENCPARCELASFATU"])
            ));

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function editProposalModel($params)
    {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->Environment->getNrOrg();
            $cdoperador = $this->Environment->getCdOperador();

            $this->ProposalModelFactory->editProposalModel(array(
                "NRORCAMENTOMODELO" => $params["NRORCAMENTOMODELO"],
                "IDATIVO" => $params["IDATIVO"],
                "IDBLOQALTERADADOS" => $params["IDBLOQALTERADADOS"],
                "NMORCAMENTOMODELO" => $params["NMORCAMENTOMODELO"],
                "QTDIASVALIDADE" => $params["QTDIASVALIDADE"],
                "QTDIASCONTRATO" => $params["QTDIASCONTRATO"],
                "IDUTILIZACOMP" => $params["IDUTILIZACOMP"],
                "NRDIASCARENCIA" => $params["NRDIASCARENCIA"],
                "IDOBRIGAOPORTUNIDADE" => $params["IDOBRIGAOPORTUNIDADE"],
                "NRSEQMODCONTRA" => $params["NRSEQMODCONTRA"],
                "CDMINEMORCA" => $params["CDMINEMORCA"],
                "IDUTILIZAREVENDA" => $params["IDUTILIZAREVENDA"],
                "DSREFERENCIAORCA" => $params["DSREFERENCIAORCA"],
                "DSOBSERVACAOORCA" => $params["DSOBSERVACAOORCA"],
                "IDASSELETRDIGITAL" => $params["IDASSELETRDIGITAL"],
                "IDENVIADOCCONCLUSAO" => $params["IDENVIADOCCONCLUSAO"],
                "NRTIPODOCCERTSIGN" => $params["NRTIPODOCCERTSIGN"],
                "IDENVIADOCOPERRESPCONCASS" => $params["IDENVIADOCOPERRESPCONCASS"],
                "NRORG" => $nrorg,
                "CDOPERULTATU" => $cdoperador,
            ));
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
}
